@@ -250,6 +250,7 @@ NotationScene::getCurrentStaff()
 void
 NotationScene::setCurrentStaff(NotationStaff *staff)
 {
+    if (! staff) return;
     // To unallow the direct edition of a repeated segment do it never be
     // the current one
     if (m_showRepeated && !m_editRepeated) {
@@ -262,10 +263,34 @@ NotationScene::setCurrentStaff(NotationStaff *staff)
                 m_currentStaff = i;
                 emit currentStaffChanged();
                 emit currentViewSegmentChanged(staff);
+                break;
             }
-            return;
         }
     }
+    // update highlighting
+    NotationStaff* currentStaff = getCurrentStaff();
+    Segment* currentSegment = &(currentStaff->getSegment());
+    TrackId currentTrack = currentSegment->getTrack();
+
+    RG_DEBUG << "highlight current" <<
+        currentStaff << currentSegment << currentTrack;
+    for (int i = 0; i < int(m_staffs.size()); ++i) {
+        NotationStaff* iStaff = m_staffs[i];
+        Segment* iSegment = &(iStaff->getSegment());
+        TrackId iTrack = iSegment->getTrack();
+        bool onSameTrack = (currentTrack == iTrack);
+        RG_DEBUG << "highlight iter" << iStaff << iSegment << onSameTrack;
+        bool highlight = true;
+        if (iSegment != currentSegment && onSameTrack &&
+            m_highlightMode == "highlight_current_on_track") highlight = false;
+        if (iStaff != currentStaff &&
+            m_highlightMode == "highlight_current") highlight = false;
+        // do not affect repeats
+        if (iSegment->isTmp()) highlight = true;
+
+        RG_DEBUG << "highlight staff" << highlight;
+        m_staffs[i]->setHighlight(highlight);
+    }
 }
 
 void
@@ -384,7 +409,6 @@ NotationScene::setStaffs(RosegardenDocument *document,
         initCurrentStaffIndex();
     }
 
-
     connect(CommandHistory::getInstance(), &CommandHistory::commandExecuted,
             this, &NotationScene::slotCommandExecuted);
 }
@@ -1099,29 +1123,30 @@ NotationScene::getInsertionTime(bool allowEndTime) const
 NotationScene::CursorCoordinates
 NotationScene::getCursorCoordinates(timeT t) const
 {
-    if (m_staffs.empty() || !m_hlayout) return CursorCoordinates();
+    if (m_staffs.empty() || !m_hlayout)
+        return CursorCoordinates();
 
-    NotationStaff *topStaff = nullptr;
-    NotationStaff *bottomStaff = nullptr;
+    const NotationStaff *topStaff = nullptr;
+    const NotationStaff *bottomStaff = nullptr;
+
+    // Find the topmost and bottom-most staves.
     for (uint i = 0; i < m_staffs.size(); ++i) {
-        if (!m_staffs[i]) continue;
-        if (!topStaff || m_staffs[i]->getY() < topStaff->getY()) {
-            topStaff = m_staffs[i];
-        }
-        if (!bottomStaff || m_staffs[i]->getY() > bottomStaff->getY()) {
-            bottomStaff = m_staffs[i];
-        }
+        const NotationStaff *staff = m_staffs[i];
+        if (!staff)
+            continue;
+
+        // If first or higher, save it.
+        if (!topStaff  ||  staff->getY() < topStaff->getY())
+            topStaff = staff;
+        // If first or lower, save it.
+        if (!bottomStaff  ||  staff->getY() > bottomStaff->getY())
+            bottomStaff = staff;
     }
 
-    NotationStaff *currentStaff = nullptr;
-    if (m_currentStaff < (int)m_staffs.size()) {
-        currentStaff = m_staffs[m_currentStaff];
-    }
+    const timeT snapped = snapTimeToNoteBoundary(t, true);
 
-    timeT snapped = snapTimeToNoteBoundary(t, true);
-
-    double x = m_hlayout->getXForTime(t);
-    double sx = m_hlayout->getXForTimeByEvent(snapped);
+    const double x = m_hlayout->getXForTime(t);
+    const double sx = m_hlayout->getXForTimeByEvent(snapped);
 
     StaffLayout::StaffLayoutCoords top =
         topStaff->getSceneCoordsForLayoutCoords
@@ -1134,6 +1159,10 @@ NotationScene::getCursorCoordinates(timeT t) const
     StaffLayout::StaffLayoutCoords singleTop = top;
     StaffLayout::StaffLayoutCoords singleBottom = bottom;
 
+    const NotationStaff *currentStaff = nullptr;
+    if (m_currentStaff < (int)m_staffs.size())
+        currentStaff = m_staffs[m_currentStaff];
+
     if (currentStaff) {
         singleTop =
             currentStaff->getSceneCoordsForLayoutCoords
@@ -1146,8 +1175,20 @@ NotationScene::getCursorCoordinates(timeT t) const
     CursorCoordinates cc;
     cc.allStaffs = QLineF(top.first, top.second,
                           bottom.first, bottom.second);
-    cc.currentStaff = QLineF(singleTop.first, singleTop.second,
-                             singleBottom.first, singleBottom.second);
+
+    // if the time is within the current segment we take the
+    // singleTop/Bottom cursor for time accuracy within that
+    // segment. If the time is outside the current segment we just use
+    // the top/bottom x coordinate to avoid the cursor jumping to the
+    // current segment.
+    cc.currentStaff = QLineF(top.first, singleTop.second,
+                             bottom.first, singleBottom.second);
+    if (currentStaff && currentStaff->includesTime(t)) {
+        // take the cursor position caluclated from the layout
+        cc.currentStaff = QLineF(singleTop.first, singleTop.second,
+                                 singleBottom.first, singleBottom.second);
+    }
+
     return cc;
 }
 
@@ -2233,6 +2274,14 @@ NotationScene::updatePageSize()
     layout(nullptr, 0, 0);
 }
 
+void NotationScene::setHighlightMode(const QString& highlightMode)
+{
+    RG_DEBUG << "setHighlightMode" << highlightMode;
+    // update highlighting
+    m_highlightMode = highlightMode;
+    setCurrentStaff(getCurrentStaff());
+}
+
 ///YG: Only for debug
 void
 NotationScene::dumpBarDataMap()
@@ -2240,5 +2289,38 @@ NotationScene::dumpBarDataMap()
     m_hlayout->dumpBarDataMap();
 }
 
+#if 0
+void
+NotationScene::setCurrentStaff(const timeT t)
+{
+    // ??? This is an attempt to fix Bug #1672.  It is currently not being
+    //     used as it introduces new problems.  Need to come up with
+    //     a better solution if possible.
+    //
+    //     See NotationWidget::updatePointer() which has a commented out
+    //     call to this.
+
+    NotationStaff *currentStaff = getCurrentStaff();
+
+    // Get the pointer scene coords for t (regardless of staff).
+    const double pointerSX = m_hlayout->getXForTime(t);
+    // To avoid vertical jumps, use the current staff's Y.
+    const double pointerSY = currentStaff->getY();
+    // figure out which staff that is in
+    // ??? Does this do "nearest"?  Probably need to test some
+    //     wild staff arrangements to make sure this works for all.
+    NotationStaff *staff =
+            getStaffForSceneCoords(pointerSX, pointerSY);
+    // If this is a different staff from the current...
+    if (staff != currentStaff) {
+        // set this as the current staff
+        setCurrentStaff(staff);
+        // ??? Should we change the selection to indicate new current staff?
+        //     That's NotationView::slotEditSelectWholeStaff() that we
+        //     usually use.  So the caller will likely need to do that.
+    }
+}
+#endif
+
 
 }
